@@ -43,7 +43,7 @@
 #include "crit.h"
 #include "utils.h"
 
-static char *SCCSid = "@(#)cygrunsrv V0.99, " __DATE__ "\n";
+static char *SCCSid = "@(#)cygrunsrv V1.0, " __DATE__ "\n";
 
 struct option longopts[] = {
   { "install", required_argument, NULL, 'I' },
@@ -397,9 +397,10 @@ get_reg_entries (const char *name, char *&path, char *&args, char *&dir,
   if ((ret = RegOpenKeyEx (HKEY_LOCAL_MACHINE, param_key, 0,
   			   KEY_READ, &env_key)) == ERROR_SUCCESS)
     {
-      char kname[256];
-      char kvalue[256];
-      DWORD ksize;
+      /* Max. size of value names is 16K since NT 5.1 (XP), 260 otherwise. */
+      char kname[16384];
+      char *kvalue;
+      DWORD type, ksize;
       int idx;
 
       if (!(env = (env_t *) calloc (MAX_ENV + 1,  sizeof (env_t))))
@@ -410,20 +411,29 @@ get_reg_entries (const char *name, char *&path, char *&args, char *&dir,
       idx = 0;
       for (int i = 0; i <= MAX_ENV + 2; ++i)
 	{
-	  ret = RegEnumValue (env_key, i, kname, (size = 256, &size), 0,
-			      NULL, (BYTE *) kvalue, (ksize = 256, &ksize));
+	  ret = RegEnumValue (env_key, i, kname, (size = 16384, &size), 0,
+			      &type, NULL, (ksize = 0, &ksize));
 	  if (ret == ERROR_NO_MORE_ITEMS)
 	    break;
 	  if (ret != ERROR_SUCCESS && ret != ERROR_MORE_DATA)
 	    goto out;
-	  if (!(env[idx].name = strdup (kname)) ||
-	      !(env[idx].val = strdup (kvalue)))
+	  if (type != REG_SZ)
+	    continue;
+	  if (!(kvalue = (char *) malloc (ksize)))
 	    {
 	      ret = ERROR_OUTOFMEMORY;
 	      goto out;
 	    }
-	  strcpy (env[idx].name, kname);
-	  strcpy (env[idx].val, kvalue);
+	  ret = RegEnumValue (env_key, i, kname, (size = 16384, &size), 0,
+			      NULL, (BYTE *) kvalue, &ksize);
+	  if (ret != ERROR_SUCCESS)
+	    goto out;
+	  if (!(env[idx].name = strdup (kname)))
+	    {
+	      ret = ERROR_OUTOFMEMORY;
+	      goto out;
+	    }
+	  env[idx].val = kvalue;
 	  ++idx;
 	}
       if (!idx)
